public with sharing class EISDashboardController {
    
    // Date From
    public Date dateFrom {get; set;}
    
    // Date To
    public Date dateTo {get; set;}
    
    // Policy Type picklist values
    public List<selectOption> policyTypeOptions {get; set;}
    
    // selected value in picklist
    public string selectedPolicyType {get; set;}
    
    // Terminating Renewal Totals
    public decimal totalTerminatingRenewalsPolicyAmount {get; set;}
    public decimal totalTerminatingRenewalsPolicyCount  {get; set;}
    
    // Successful Renewal Totals
    public decimal totalSuccessfulRenewalsPolicyAmount {get; set;}
    public decimal totalSuccessfulRenewalsPolicyCount  {get; set;}    

    // Unsuccessful Renewal Totals
    public decimal totalUnsuccessfulRenewalsPolicyAmount {get; set;}
    public decimal totalUnsuccessfulRenewalsPolicyCount  {get; set;}
    
    // Renewal Success Rate
    public decimal renewalSuccessRatePolicyAmount { get; set;}    
    public decimal renewalSuccessRatePolicyCount  { get; set;}
    
    // Total Policies Won
    public decimal totalPoliciesWonPolicyAmount { get; set;}    
    public decimal totalPoliciesWonPolicyCount  { get; set;}
    
    // Policy Growth Rate
    public decimal policyGrowthRatePolicyAmount { get; set;}    
    public decimal policyGrowthRatePolicyCount  { get; set;}    
        
    // New Policy Request Totals
    public decimal totalNewPolicyRequestAmount {get; set;}
    public decimal totalNewPolicyRequestCount  {get; set;}
    
    // New Policies Won Totals
    public decimal totalNewPolicyRequestWonAmount {get; set;}
    public decimal totalNewPolicyRequestWonCount  {get; set;}
    
    // New Policy Request Quoted Totals
    public decimal totalNewPolicyRequestQuotedAmount {get; set;}
    public decimal totalNewPolicyRequestQuotedCount  {get; set;}

    // New Policy Quote Rate
    public decimal newPolicyQuoteRateAmount { get; set;}    
    public decimal newPolicyQuoteRateCount  { get; set;}
    
    // New Policy Success Rate
    public decimal newPolicySuccessRateAmount { get; set;}    
    public decimal newPolicySuccessRateCount  { get; set;}
    
    // Chart Data Series for Renewals Charts
    public List<ChartData> renewalPieData {get; set;}
    public List<ChartData> renewalGaugeData {get; set;}  
    
    // Chart Data for New Policy Charts
    public List<ChartData> newPolicyGaugeData {get; set;}
    public List<ChartData> newPolicyPieData {get; set;}
         
    
    public EISDashboardController() {
    
        // initialize values for "Policy Type" picklist
        policyTypeOptions = new List<selectOption>();
        policyTypeOptions.add(new selectOption('All','All'));
        List<Schema.PicklistEntry> policyTypes = EIS_Policy__c.fields.Policy_Type__c.getDescribe().getpicklistvalues();

		for (Schema.PicklistEntry a : policyTypes) {
			policyTypeOptions.add(new selectOption(a.getValue(), a.getLabel()));
		}
		
		// set initial value for Policy Type to 'All'
		selectedPolicyType = 'All';
    
    	// initialize dashboard values
    	initializeEISDashboard();
               
    }
    
    // start EIS Dashboard generation
    public PageReference startGenerateEISDashboard() {
    	if (dateFrom != null && dateTo != null) {
    		if (dateTo >= dateFrom) {
    			generateEISDashboard();
    		} else {
    			// display custom page message
    			ApexPages.Message myMsg = new ApexPages.Message(ApexPages.Severity.ERROR,'Date To must be greater than or equal to Date From!');
            	ApexPages.addMessage(myMsg);
    		}
    	} else {
    		// display custom page message
    		ApexPages.Message myMsg = new ApexPages.Message(ApexPages.Severity.ERROR,'You must select a Date From and a Date To!');
            ApexPages.addMessage(myMsg);
    	}
    	return null;
    }
    
    // set dashboard values to zero in preparation for generation
    private void initializeEISDashboard() {
    	
    	// initialize controller values to zero
        totalTerminatingRenewalsPolicyAmount = 0;
        totalTerminatingRenewalsPolicyCount = 0;
        totalSuccessfulRenewalsPolicyAmount = 0;
        totalSuccessfulRenewalsPolicyCount = 0;
        totalUnsuccessfulRenewalsPolicyAmount = 0;
        totalUnsuccessfulRenewalsPolicyCount = 0;
        renewalSuccessRatePolicyAmount = 0;
        renewalSuccessRatePolicyCount = 0;
        totalPoliciesWonPolicyAmount = 0;
        totalPoliciesWonPolicyCount = 0;
        policyGrowthRatePolicyAmount = 0;
        policyGrowthRatePolicyCount = 0;
        totalNewPolicyRequestAmount = 0;
        totalNewPolicyRequestCount = 0;
        totalNewPolicyRequestWonAmount = 0;
        totalNewPolicyRequestWonCount = 0;
        totalNewPolicyRequestQuotedAmount = 0;
        totalNewPolicyRequestQuotedCount = 0;
        newPolicyQuoteRateAmount = 0;
        newPolicyQuoteRateCount = 0;
        newPolicySuccessRateAmount = 0;
        newPolicySuccessRateCount = 0;
    }
    
    // generate the dashboard
    private void generateEISDashboard() {
    	initializeEISDashboard();
    	generateRenewals();
    	generateNewPolicies();
    	generateSuccessIndicators();
    	generateCharts();
    }
    
    // generic function to calculate totals from an AggregateResult
    private Map<string, decimal> calculateAggregateTotals(List<AggregateResult> results,
    				string amountString, string countString) {
    											
    	decimal policyAmount = 0;
    	decimal policyCount = 0;
    	
    	// Loop through aggregate results returned and get totals
        for (AggregateResult a : results) {
        	if (selectedPolicyType == 'All') {
        		policyAmount = decimal.valueOf(String.valueOf(a.get(amountString)));
        		policyCount = decimal.valueOf(String.valueOf(a.get(countString)));
        	} else {
        		if (String.valueOf(a.get('Policy_Type__c')) == selectedPolicyType) {
        			policyAmount = decimal.valueOf(String.valueOf(a.get(amountString)));
        			policyCount = decimal.valueOf(String.valueOf(a.get(countString)));
        		}
        	}
        }
        
        // construct Map and return it
        Map<string, decimal> resultMap = new Map<string, decimal>();
        resultMap.put(amountString, policyAmount);
        resultMap.put(countString, policyCount);
        
        return resultMap;
 
    }
    
    // generate Renewals figures
    private void generateRenewals() {
        
        Map<string, decimal> renewalQueryTotals = new Map<string, decimal>();
        
        // get amounts for Terminating Renewals
        List<AggregateResult> renewalsAggregateResult = 
            [SELECT SUM(Policy_Amount__c) totalRenewalsAmount, 
             COUNT(Id) totalRenewalsCount, Policy_Type__c
             FROM EIS_Policy__c
             WHERE Renewal__c = true
             AND Policy_Date__c >= :dateFrom AND Policy_Date__c <= :dateTo
             GROUP BY Policy_Type__c];
              
        // get the Terminating Renewal totals
        renewalQueryTotals = calculateAggregateTotals(renewalsAggregateResult, 
        														'totalRenewalsAmount',
        														'totalRenewalsCount');
        totalTerminatingRenewalsPolicyAmount = renewalQueryTotals.get('totalRenewalsAmount');
        totalTerminatingRenewalsPolicyCount = renewalQueryTotals.get('totalRenewalsCount');
                														
             
        // get amounts for Successful Terminating Renewals
        List<AggregateResult> successfulRenewalsAggregateResult = 
            [SELECT SUM(Policy_Amount__c) successfulRenewalsAmount, 
             COUNT(Id) successfulRenewalsCount, Policy_Type__c 
             FROM EIS_Policy__c
             WHERE Renewal__c = true
             AND Status__c = 'Won'
             AND Policy_Date__c >= :dateFrom AND Policy_Date__c <= :dateTo
             GROUP BY Policy_Type__c];
             
        // get the successful Terminating Renewals Totals
        renewalQueryTotals = calculateAggregateTotals(successfulRenewalsAggregateResult, 
        														'successfulRenewalsAmount',
        														'successfulRenewalsCount');
        totalSuccessfulRenewalsPolicyAmount = renewalQueryTotals.get('successfulRenewalsAmount');
        totalSuccessfulRenewalsPolicyCount = renewalQueryTotals.get('successfulRenewalsCount');     

        
        // use a calculation to get unsuccessful renewals totals so we don't have to access Database
        totalUnsuccessfulRenewalsPolicyAmount = totalTerminatingRenewalsPolicyAmount - totalSuccessfulRenewalsPolicyAmount;
        totalUnsuccessfulRenewalsPolicyCount = totalTerminatingRenewalsPolicyCount - totalSuccessfulRenewalsPolicyCount;

    }
    
    
    // Generate New Policy Figures
    private void generateNewPolicies() {
    	
    	Map<string, decimal> newPolicyQueryTotals = new Map<string, decimal>();
    	
        // get amounts for New Policies
        List<AggregateResult> newPolicyAggregateResult = 
            [SELECT SUM(Policy_Amount__c) totalNewPolicyAmount, 
             COUNT(Id) totalNewPolicyCount, Policy_Type__c 
             FROM EIS_Policy__c
             WHERE Renewal__c = false
             AND Policy_Date__c >= :dateFrom AND Policy_Date__c <= :dateTo
             GROUP BY Policy_Type__c];
             
        // get the New Policy Totals
        newPolicyQueryTotals = calculateAggregateTotals(newPolicyAggregateResult, 
        														'totalNewPolicyAmount',
        														'totalNewPolicyCount');
        totalNewPolicyRequestAmount = newPolicyQueryTotals.get('totalNewPolicyAmount');
        totalNewPolicyRequestCount = newPolicyQueryTotals.get('totalNewPolicyCount');
        
       
        // get amounts for New Policies Won
        List<AggregateResult> newPolicyWonAggregateResult = 
            [SELECT SUM(Policy_Amount__c) totalNewPolicyWonAmount, 
             COUNT(Id) totalNewPolicyWonCount, Policy_Type__c 
             FROM EIS_Policy__c
             WHERE Renewal__c = false
             AND Status__c = 'Won'
             AND Policy_Date__c >= :dateFrom AND Policy_Date__c <= :dateTo
             GROUP BY Policy_Type__c];

        // get the New Policy Totals
        newPolicyQueryTotals = calculateAggregateTotals(newPolicyWonAggregateResult, 
        														'totalNewPolicyWonAmount',
        														'totalNewPolicyWonCount');
        totalNewPolicyRequestWonAmount = newPolicyQueryTotals.get('totalNewPolicyWonAmount');
        totalNewPolicyRequestWonCount = newPolicyQueryTotals.get('totalNewPolicyWonCount');

        
        // get amounts for New Policies Quoted
        List<AggregateResult> newPolicyQuotedAggregateResult = 
            [SELECT SUM(Policy_Amount__c) totalNewPolicyQuotedAmt, 
             COUNT(Id) totalNewPolicyQuotedCount, Policy_Type__c 
             FROM EIS_Policy__c
             WHERE Renewal__c = false
             AND Quoted__c = true
             AND Policy_Date__c >= :dateFrom AND Policy_Date__c <= :dateTo
             GROUP BY Policy_Type__c];

        // get the New Policy Totals
        newPolicyQueryTotals = calculateAggregateTotals(newPolicyQuotedAggregateResult, 
        														'totalNewPolicyQuotedAmt',
        														'totalNewPolicyQuotedCount');
        totalNewPolicyRequestQuotedAmount = newPolicyQueryTotals.get('totalNewPolicyQuotedAmt');
        totalNewPolicyRequestQuotedCount = newPolicyQueryTotals.get('totalNewPolicyQuotedCount');
        
    }
    
    // Generate Success Indicators
    private void generateSuccessIndicators() {
        
        // Renewals Success Rate
        if (totalTerminatingRenewalsPolicyAmount != 0 && totalSuccessfulRenewalsPolicyAmount != 0) {
            renewalSuccessRatePolicyAmount = (totalSuccessfulRenewalsPolicyAmount / totalTerminatingRenewalsPolicyAmount) * 100;
            renewalSuccessRatePolicyCount = (totalSuccessfulRenewalsPolicyCount / totalTerminatingRenewalsPolicyCount) * 100;
        }
        
        // Total Policies Won
        totalPoliciesWonPolicyAmount = totalTerminatingRenewalsPolicyAmount - totalUnsuccessfulRenewalsPolicyAmount
                                                + totalNewPolicyRequestWonAmount;
        totalPoliciesWonPolicyCount = totalTerminatingRenewalsPolicyCount - totalUnsuccessfulRenewalsPolicyCount
                                                + totalNewPolicyRequestWonCount; 
                                                
        // Policy Growth Rate
        if (totalTerminatingRenewalsPolicyAmount != 0 && totalPoliciesWonPolicyAmount != 0) {
            policyGrowthRatePolicyAmount = ((totalPoliciesWonPolicyAmount - totalTerminatingRenewalsPolicyAmount)
                                                / totalTerminatingRenewalsPolicyAmount) * 100;
            policyGrowthRatePolicyCount = ((totalPoliciesWonPolicyCount - totalTerminatingRenewalsPolicyCount)
                                                / totalTerminatingRenewalsPolicyCount) * 100;
        }
        
        // New Policy Quoted Rate
        if (totalNewPolicyRequestQuotedAmount != 0 && totalNewPolicyRequestAmount != 0) {
            newPolicyQuoteRateAmount = (totalNewPolicyRequestQuotedAmount / totalNewPolicyRequestAmount) * 100;
            newPolicyQuoteRateCount = (totalNewPolicyRequestQuotedCount / totalNewPolicyRequestCount) * 100;
        }
        
        // New Policy Success Rate
        if (totalNewPolicyRequestWonAmount !=0 && totalNewPolicyRequestAmount !=0) {
            newPolicySuccessRateAmount = (totalNewPolicyRequestWonAmount / totalNewPolicyRequestAmount) * 100;
            newPolicySuccessRateCount = (totalNewPolicyRequestWonCount / totalNewPolicyRequestCount) * 100;
        }
   
    }
    
    private void generateCharts() {
    	
    	// Renewals Pie Chart
    	renewalPieData = new List<ChartData>();
    	renewalPieData.add(new ChartData('Successful $',totalSuccessfulRenewalsPolicyAmount));
    	renewalPieData.add(new ChartData('Unsuccessful $',totalUnsuccessfulRenewalsPolicyAmount));
    	
    	// Renewals Success Rate Gauge
    	renewalGaugeData = new List<ChartData>();
    	renewalGaugeData.add(new ChartData('Success Rate %', renewalSuccessRatePolicyAmount));
    	
    	// New Policy Success Rate Gauge
    	newPolicyGaugeData = new List<ChartData>();
    	newPolicyGaugeData.add(new ChartData('Success Rate %', newPolicySuccessRateAmount));
    	
    	// New Policy Quoted Amount Pie Chart
    	newPolicyPieData = new List<ChartData>();
    	newPolicyPieData.add(new ChartData('Quoted $', totalNewPolicyRequestQuotedAmount));
    	newPolicyPieData.add(new ChartData('Not Quoted $', totalNewPolicyRequestAmount
    													- totalNewPolicyRequestQuotedAmount)); 
    	
    }
    
    // Wrapper class for Chart Data
    public class ChartData {

        public String name { get; set; }
        public Integer data { get; set; }

        public ChartData(String name, Integer data) {
            this.name = name;
            this.data = data;
        }
        
        public ChartData(String name, decimal data) {
            this.name = name;
            this.data = integer.valueOf(data);
        }
    }

}