@isTest
private class testEISDashboardController {
    
    static private integer MAX_RECORD_BATCH_SIZE = 5; 
    static Date policyDate = System.Today();
    static Date policyDateTo = System.Today() + 10;
    
    static List<EIS_Policy__c> createTestData() {
        
        List<EIS_Policy__c> testPolicies = new List<EIS_Policy__c>();
        
        // RENEWALS (Won)
        for (integer i=0; i < MAX_RECORD_BATCH_SIZE; i++) {
            EIS_Policy__c eisPolicy = new EIS_Policy__c(
                Policy_Amount__c = 1000, 
                Policy_Date__c = policyDate,
                Policy_Type__c = 'Motor Vehicle',
                Quoted__c = false,
                Renewal__c = true,
                Status__c = 'Won');
            
            testPolicies.add(eisPolicy);    
        }
        
        // RENEWALS (Lost)
        for (integer i=0; i < MAX_RECORD_BATCH_SIZE; i++) {
            EIS_Policy__c eisPolicy = new EIS_Policy__c(
                Policy_Amount__c = 500, 
                Policy_Date__c = policyDate,
                Policy_Type__c = 'Motor Vehicle',
                Quoted__c = false,
                Renewal__c = true,
                Status__c = 'Lost');
            
            testPolicies.add(eisPolicy);    
        }
        
        // NEW POLICY WON (NO QUOTE)
        for (integer i=0; i < MAX_RECORD_BATCH_SIZE; i++) {
            EIS_Policy__c eisPolicy = new EIS_Policy__c(
                Policy_Amount__c = 1000, 
                Policy_Date__c = policyDate,
                Policy_Type__c = 'Motor Vehicle',
                Quoted__c = false,
                Renewal__c = false,
                Status__c = 'Won');
            
            testPolicies.add(eisPolicy);    
        }
        
        // NEW POLICY LOST (NO QUOTE)
        for (integer i=0; i < MAX_RECORD_BATCH_SIZE; i++) {
            EIS_Policy__c eisPolicy = new EIS_Policy__c(
                Policy_Amount__c = 500, 
                Policy_Date__c = policyDate,
                Policy_Type__c = 'Motor Vehicle',
                Quoted__c = false,
                Renewal__c = false,
                Status__c = 'Lost');
            
            testPolicies.add(eisPolicy);    
        }
        
        // NEW POLICY WON (QUOTED)
        for (integer i=0; i < MAX_RECORD_BATCH_SIZE; i++) {
            EIS_Policy__c eisPolicy = new EIS_Policy__c(
                Policy_Amount__c = 1000, 
                Policy_Date__c = policyDate,
                Policy_Type__c = 'Motor Vehicle',
                Quoted__c = true,
                Renewal__c = false,
                Status__c = 'Won');
            
            testPolicies.add(eisPolicy);    
        }
        
        // NEW POLICY LOST (QUOTED)
        for (integer i=0; i < MAX_RECORD_BATCH_SIZE; i++) {
            EIS_Policy__c eisPolicy = new EIS_Policy__c(
                Policy_Amount__c = 500, 
                Policy_Date__c = policyDate,
                Policy_Type__c = 'Motor Vehicle',
                Quoted__c = true,
                Renewal__c = false,
                Status__c = 'Lost');
            
            testPolicies.add(eisPolicy);    
        }
                     
        return testPolicies;
                
    }
    
    // Test that the EIS Dashboard Page Controller is calculating reporting values correctly
    static testMethod void testEISDashboard() {
        
        List<EIS_Policy__c> testDataPolicies = createTestData();
        
        System.AssertEquals(testDataPolicies.size(), 30);
        
        Test.startTest();
        
        // insert data
        insert testDataPolicies;
        
        // get reference to EIS Dashboard Page
        PageReference EISDashboard = Page.EIS_Dashboard_Page;
        Test.setCurrentPage(EISDashboard);
        
        // get a Page controller instance
        EISDashboardController eisDashboardController = new EISDashboardController();
        
        // Set Date From and Date To on Controller
        Date controllerDateFrom = policyDate;
        Date controllerDateTo = policyDateTo;
        eisDashboardController.datefrom = controllerDateFrom;
        eisDashboardController.dateTo = controllerDateTo;
        System.AssertEquals(eisDashboardController.dateFrom, controllerDateFrom);
        System.AssertEquals(eisDashboardController.dateTo, controllerDateTo);
        
        // Set the Policy Type on Controller
        
        // Toggle to 'All' to test total for all Policies
        eisDashboardController.selectedPolicyType = 'All';
        System.assertEquals(eisDashboardController.selectedPolicyType, 'All');
        
        // Toggle to 'Motor Vehicle' to test a Policy Type
        //eisDashboardController.selectedPolicyType = 'Motor Vehicle';
        //System.assertEquals(eisDashboardController.selectedPolicyType, 'Motor Vehicle');
        
        // generate EIS Dashboard
        eisDashboardController.startGenerateEISDashboard();
        
        // Test that correct terminating renewal totals are returned
        System.debug(LoggingLevel.DEBUG, '***Total Terminating Renewals Policy Amount = $' 
                            + eisDashboardController.totalTerminatingRenewalsPolicyAmount);
        System.debug(LoggingLevel.DEBUG, '***Total Terminating Renewals Policy Count = ' 
                            + eisDashboardController.totalTerminatingRenewalsPolicyCount);                    
        System.AssertEquals(eisDashboardController.totalTerminatingRenewalsPolicyAmount, 7500);
        System.AssertEquals(eisDashboardController.totalTerminatingRenewalsPolicyCount, 10);
        
        // Test that correct successful renewal totals are returned
        System.debug(LoggingLevel.DEBUG, '***Total Successful Renewals Policy Amount = $' 
                            + eisDashboardController.totalSuccessfulRenewalsPolicyAmount);
        System.debug(LoggingLevel.DEBUG, '***Total Successful Renewals Policy Count = ' 
                            + eisDashboardController.totalSuccessfulRenewalsPolicyCount);                    
        System.AssertEquals(eisDashboardController.totalSuccessfulRenewalsPolicyAmount, 5000);
        System.AssertEquals(eisDashboardController.totalSuccessfulRenewalsPolicyCount, 5);
        
        // Test that correct unsuccessful renewal totals are returned
        System.debug(LoggingLevel.DEBUG, '***Total Unsuccessful Renewals Policy Amount = $' 
                            + eisDashboardController.totalUnsuccessfulRenewalsPolicyAmount);
        System.debug(LoggingLevel.DEBUG, '***Total Unsuccessful Renewals Policy Count = ' 
                            + eisDashboardController.totalUnsuccessfulRenewalsPolicyCount);                    
        System.AssertEquals(eisDashboardController.totalUnsuccessfulRenewalsPolicyAmount, 2500);
        System.AssertEquals(eisDashboardController.totalUnsuccessfulRenewalsPolicyCount, 5);
                
        // Test that the correct New Policy totals are returned
        System.debug(LoggingLevel.DEBUG, '***Total New Policy Amount = $' 
                            + eisDashboardController.totalNewPolicyRequestAmount);
        System.debug(LoggingLevel.DEBUG, '***Total New Policy Count = ' 
                            + eisDashboardController.totalNewPolicyRequestCount);                    
        System.AssertEquals(eisDashboardController.totalNewPolicyRequestAmount, 15000);
        System.AssertEquals(eisDashboardController.totalNewPolicyRequestCount, 20);        

        // Test that the correct New Policies Won totals are returned
        System.debug(LoggingLevel.DEBUG, '***Total New Policies Won Amount = $' 
                            + eisDashboardController.totalNewPolicyRequestWonAmount);
        System.debug(LoggingLevel.DEBUG, '***Total New Policies Won Count = ' 
                            + eisDashboardController.totalNewPolicyRequestWonCount);                    
        System.AssertEquals(eisDashboardController.totalNewPolicyRequestWonAmount, 10000);
        System.AssertEquals(eisDashboardController.totalNewPolicyRequestWonCount, 10);  
        
        // Test that the correct New Policies Quoted totals are returned
        System.debug(LoggingLevel.DEBUG, '***Total New Policies Quoted Amount = $' 
                            + eisDashboardController.totalNewPolicyRequestQuotedAmount);
        System.debug(LoggingLevel.DEBUG, '***Total New Policies Quoted Count = ' 
                            + eisDashboardController.totalNewPolicyRequestQuotedCount);                    
        System.AssertEquals(eisDashboardController.totalNewPolicyRequestQuotedAmount, 7500);
        System.AssertEquals(eisDashboardController.totalNewPolicyRequestQuotedCount, 10);
                
        // Test that the correct Renewal Success Rates totals are returned
        System.debug(LoggingLevel.DEBUG, '***Total Renewal Success Rate (Amount) = %' 
                            + eisDashboardController.renewalSuccessRatePolicyAmount);
        System.debug(LoggingLevel.DEBUG, '***Total Renewal Success Rate (Count) = %' 
                            + eisDashboardController.renewalSuccessRatePolicyCount);                    
        System.AssertEquals(eisDashboardController.renewalSuccessRatePolicyAmount.round(System.RoundingMode.DOWN), 66);
        System.AssertEquals(eisDashboardController.renewalSuccessRatePolicyCount.round(System.RoundingMode.DOWN), 50);                    

        // Test that the correct Total Policies Won totals are returned
        System.debug(LoggingLevel.DEBUG, '***Total Policies Won (Amount) = $' 
                            + eisDashboardController.totalPoliciesWonPolicyAmount);
        System.debug(LoggingLevel.DEBUG, '***Total Policies Won (Count) = ' 
                            + eisDashboardController.totalPoliciesWonPolicyCount);                    
        System.AssertEquals(eisDashboardController.totalPoliciesWonPolicyAmount, 15000);
        System.AssertEquals(eisDashboardController.totalPoliciesWonPolicyCount, 15); 
        
       // Test that the correct Policy Growth Rates totals are returned
        System.debug(LoggingLevel.DEBUG, '***Total Policy Growth Rate (Amount) = %' 
                            + eisDashboardController.policyGrowthRatePolicyAmount);
        System.debug(LoggingLevel.DEBUG, '***Total Policy Growth Rate (Count) = %' 
                            + eisDashboardController.policyGrowthRatePolicyCount);                    
        System.AssertEquals(eisDashboardController.policyGrowthRatePolicyAmount.round(System.RoundingMode.DOWN), 100);
        System.AssertEquals(eisDashboardController.policyGrowthRatePolicyCount.round(System.RoundingMode.DOWN), 50);
        
        // Test that the correct New Policy Quote Rates are returned
        System.debug(LoggingLevel.DEBUG, '***Total New Policy Quote Rate (Amount) = %' 
                            + eisDashboardController.newPolicyQuoteRateAmount);
        System.debug(LoggingLevel.DEBUG, '***Total New Policy Quote Rate (Count) = %' 
                            + eisDashboardController.newPolicyQuoteRateCount);                    
        System.AssertEquals(eisDashboardController.newPolicyQuoteRateAmount.round(System.RoundingMode.DOWN), 50);
        System.AssertEquals(eisDashboardController.newPolicyQuoteRateCount.round(System.RoundingMode.DOWN), 50);
                            
        // Test that the correct New Policy Success Rates are returned
        System.debug(LoggingLevel.DEBUG, '***Total New Policy Success Rate (Amount) = %' 
                            + eisDashboardController.newPolicySuccessRateAmount);
        System.debug(LoggingLevel.DEBUG, '***Total New Policy Success Rate (Count) = %' 
                            + eisDashboardController.newPolicySuccessRateCount);                    
        System.AssertEquals(eisDashboardController.newPolicySuccessRateAmount.round(System.RoundingMode.DOWN), 66);
        System.AssertEquals(eisDashboardController.newPolicySuccessRateCount.round(System.RoundingMode.DOWN), 50);
                            
                
        Test.stopTest();        
        
    }

}