/* This file initializes the AngularJS app
   and includes the necessary Force.com
   SDK Libraries
*/

console.log("***MyMobileApp.js - Initializing MyMobileApp AngularJS application...");

// initialize base AngularJS module
var app = angular.module('MyMobileApp', ['AngularForce', 'AngularForceObjectFactory', 'ngRoute']);

// get Salesforce Configuration
var SFConfig = getSalesforceConfig();

// set maximum list size
SFConfig.maxListSize = 25;

// define a constant for the AngularJS app
app.constant('SFConfig', SFConfig);


/***************************
 Configure AngularJS routes
****************************/
console.log('***MyMobileApp.js - Setting up AngularJS Routes...');
app.config(function ($routeProvider) {
	$routeProvider
	.when('/', {controller: 'HomeController', templateUrl: 'partials/home.html'})
	.when('/login', {controller: 'LoginController', templateUrl: 'partials/login.html'})
	.when('/logout', {controller: 'LogoutController', templateUrl: 'partials/logout.html'})
	.when('/callback', {controller: 'CallbackController', templateUrl: 'partials/callback.html'})
	.when('/opportunities', {controller: 'OppListController', templateUrl: 'partials/opportunity/list.html'})
	.when('/view/:oppId', {controller: 'OppViewController', templateUrl: 'partials/opportunity/view.html'})
	.when('/map', {controller: 'MapViewController', templateUrl: 'partials/map.html'})
	.otherwise({redirectTo: '/'});
});


/***********************************************
 initApp - called when authentication successful
***********************************************/
function initApp(options, forcetkClient) {
	console.log('***MyMobileApp.js - Executing initApp...');
	// initialize options object to hold Salesforce
	// access and configuration information
	options = options || {};
	options.loginUrl = SFConfig.sfLoginURL;
	options.clientId = SFConfig.consumerKey;
	options.apiVersion = 'v29.0';
	options.userAgent = 'SalesforceMobileUI/alpha';
	options.proxyUrl = options.proxyUrl || SFConfig.proxyUrl;
	
	// initialize forcetk toolkit
	Force.init(options, options.apiVersion, forcetkClient);
	console.log('***MyMobileApp.js - initApp Executed Successfully...');
}



/************************************
 Salesforce Configuration Functions
************************************/ 

function getSalesforceConfig() {
	console.log('***MyMobileApp.js - Getting Salesforce config...');
	// first check if environment configuration variables are set
    if (!configFromEnv || configFromEnv.client_id == "" || configFromEnv.client_id == "undefined"
        || configFromEnv.app_url == "" || configFromEnv.app_url == "undefined") {
        throw 'Environment variable client_id and/or app_url is missing. Please set them before you start the app';
    }
    // return config information for login URL, OAuth information and proxy URL
	return {
        sfLoginURL: 'https://login.salesforce.com/',
        consumerKey: configFromEnv.client_id,
        oAuthCallbackURL: removeTrailingSlash(configFromEnv.app_url) + '/#/callback',
        proxyUrl: removeTrailingSlash(configFromEnv.app_url) + '/proxy/'
    }
}

//Helper
function removeTrailingSlash(url) {
    return url.replace(/\/$/, "");
}

/*******************************
 Authentication Functions to
 check if user logged in, and
 log user out
*******************************/

// return authentication status
console.log('***MyMobileApp.js - Adding isLoggedIn() to app module...');
app.run(function (AngularForce, $rootScope) {
	$rootScope.isLoggedIn = function() {
		console.log('***MyMobileApp.js - checking if user logged in to Salesforce...');
		return AngularForce.authenticated();
	}	
});


// logout user
console.log('***MyMobileApp.js - Adding logout() to app module...');
app.run(function (AngularForce, $rootScope, $location) {
	$rootScope.logout = function() {
		console.log('***MyMobileApp.js - Logging User out of Salesforce...');
		AngularForce.logout();
		// Now go to logout page
		$location.path('/logout');
		//$rootScope.apply();
	}
});
