app.controller('MapViewController', function($scope, $rootScope, AngularForce, 
										$location, $routeParams, LocationService,
									    Opportunity, ReflectionService) {
	console.log("***MapViewController.js - MapController being executed...");
	
	if ($rootScope.isLoggedIn()) {
		
		var mapLocation = {
			latitude: 0,
			longitude: 0
		};
		
		var map = null;
		
		// use LocationService to get current position or default
		// co-ordinates
		LocationService.getCurrentLocation().then(function(position) {
			console.log("***MapViewController.js - Success, setting Map Location...");
			mapLocation.latitude = position.coords.latitude;
			mapLocation.longitude = position.coords.longitude;
			console.log('--> latitude:' + mapLocation.latitude);
			console.log('--> longitude: ' + mapLocation.longitude);
		}, function(defaultPosition) {
			console.log("***MapViewController.js - Unsuccessful, setting Default Map Location...");
			mapLocation.latitude = defaultPosition.latitude;
			mapLocation.longitude = defaultPosition.longitude;
			console.log('--> latitude:' + mapLocation.latitude);
			console.log('--> longitude: ' + mapLocation.longitude);
		})
		.then(function() {
			console.log('***MapViewController.js - Generating Google Maps...');
	        var mapOptions = {
	          center: new google.maps.LatLng(mapLocation.latitude, mapLocation.longitude),
	          zoom: 15,
			  mapTypeId: google.maps.MapTypeId.ROADMAP
	        };
	        map = new google.maps.Map(document.getElementById("map-canvas"),
	            mapOptions);
				
			// mark current position on map
			var marker = new google.maps.Marker({
				position: new google.maps.LatLng(mapLocation.latitude, mapLocation.longitude),
				map: map
			});	
		})
		.then(function() {
			// select any opportunities within a 5 mile radius and plot them on the map
			Opportunity.getOpportunitiesWithinRadius(mapLocation.latitude, mapLocation.longitude).then(
				function(data) {
					// success
					console.log('***MapViewController.js - Opportunity Retrieval Successful...');
					
					// LatLng Bounds Object to determine zoom level
					var bounds = new google.maps.LatLngBounds();
					// flag to determine whether to adjust default boundary
					var extendBounds = false;
					
					$.each(data, function(i, val) {
						console.log(val);
						console.log('***Opportunity Name = ' + val.Name);
						console.log('***Account Name = ' + val.Account.Name);
						// add marker to map
						var oppMarker = new google.maps.Marker({
							position: new google.maps.LatLng(val.Account.Location__Latitude__s,
												val.Account.Location__Longitude__s),
							map: map,
							icon: 'http://gmaps-samples.googlecode.com/svn/trunk/markers/green/blank.png'
						});
						// info window for the marker
						(function(i, oppMarker, val) {
							// Event listener has access to values of i and marker
							// as they were during marker creation
							google.maps.event.addListener(oppMarker, 'click', function() {
								var infoWindow = new google.maps.InfoWindow({
									content: '<strong>' + val.Name + '</strong><br />'
									+ val.Account.Name + '<br />'
									+ val.Account.BillingStreet + '<br />'
									+ val.Account.BillingCity + '<br />'
									+ val.Account.BillingState + ' '
									+ val.Account.BillingPostalCode + '<br />'
									+ val.Account.BillingCountry + '<br />'
									+ '<a href="tel:' + val.Account.Phone + '">'
									+ val.Account.Phone + '</a><br />'
								});
								infoWindow.open(map, oppMarker);
							});
						})(i, oppMarker, val);
						// Extend bounds object with each opportunity
						bounds.extend(new google.maps.LatLng(val.Account.Location__Latitude__s,
												val.Account.Location__Longitude__s));
						extendBounds = true;
					});
					// if we have at least 1 opportunity adjust the boundary to fit all
					// opportunites
					if (extendBounds) {
						map.fitBounds(bounds);
					}
				}, function(data) {
				console.log('***MapViewController.js - Error Retrieving Opportunity List...');
				console.log('readyState = ' + data.readyState);
				console.log('responseText = ' + data.responseText);
				console.log('responseJSON = ' + data.responseJSON);
				console.log('status = ' + data.status);
				console.log('statusText = ' + data.statusText);
			
			});
		});
						
	} else {
		// re-direct to login page
		$location.path('/');		
	}

});