app.factory('Opportunity', function($q, AngularForceObjectFactory, ReflectionService) {
	
	var factory = {};
	
	// get a list of opportunities
	factory.getOpportunityList = function() {
		
		var deferred = $q.defer();
		
		console.log('***Opportunity.js - Setting Up Opportunity Query...');

		// Configure Opportunity Object Criteria
		var objQuery = {
			type: 'Opportunity',
			fields: ['Id', 'Name', 'Account.Name', 'StageName', 'Amount', 'CloseDate'],
			where: "StageName <> 'Closed Won' AND StageName <> 'Closed Lost'",
			orderBy: 'Id',
			limit: 20
		};
		
		var Opportunity = AngularForceObjectFactory(objQuery);

		console.log('***Opportunity.js - Executing Opportunity Query...');
		Opportunity.query(function(data) {
			console.log('***Opportunity.js - Query Success Function Executing...');
			var opportunities = data.records;
			deferred.resolve(opportunities);
		}, function(error) {
			console.log('***Opportunity.js - Query Error Function Executing...');
			deferred.reject(error);
			
		});
				
		// return promise to controller to deliver deferred results
		return deferred.promise;
				
	};
	
	
	// get a single opportunity
	factory.getOpportunity = function(id) {
		
		var deferred = $q.defer();
		
		console.log('***Opportunity.js - Setting Up Opportunity Query...');

		// Configure Opportunity Object Criteria
		var objQuery = {
			type: 'Opportunity',
			fields: ['Id', 'Name', 'Account.Name', 'Account.Phone', 'StageName', 'Amount', 'CloseDate'],
			where: "Id = '" + id + "'",
			orderBy: 'Id',
			limit: 20
		};
		
		var Opportunity = AngularForceObjectFactory(objQuery);

		console.log('***Opportunity.js - Executing Single Opportunity Query...');
		Opportunity.query(function(data) {
			console.log('***Opportunity.js - Single Query Success Function Executing...');
			var opportunity = data.records;
			deferred.resolve(opportunity);
		}, function(error) {
			console.log('***Opportunity.js - Single Query Error Function Executing...');
			deferred.reject(error);
			
		});
				
		// return promise to controller to deliver deferred results
		return deferred.promise;
	};
	
	// get a list of opportunities within a given map radius
	factory.getOpportunitiesWithinRadius = function(currLat, currLng) {
		
		
		var deferred = $q.defer();
		
		console.log('***Opportunity.js - Setting Up Opportunity Radius Query...');

		// Configure Opportunity Object Criteria
		var objQuery = {
			type: 'Opportunity',
			fields: ['Id', 'Name', 'Account.Name', 'Account.Phone', 'Account.BillingStreet',
						'Account.BillingCity', 'Account.BillingState', 'Account.BillingCountry',
						'Account.BillingPostalCode',
						'Account.Location__Latitude__s', 'Account.Location__Longitude__s'],
			where: "DISTANCE (Account.Location__c, GEOLOCATION(" + currLat +
					"," + currLng + "),'mi') < 5",
			orderBy: 'Id',
			limit: 20
		};
		
		var Opportunity = AngularForceObjectFactory(objQuery);

		console.log('***Opportunity.js - Executing Opportunity Radius Query...');
		Opportunity.query(function(data) {
			console.log('***Opportunity.js - Radius Query Success Function Executing...');
			var reflector = new ReflectionService.Reflector(data);
			console.log('***Opportunity.js - Opportunity.query data properties:');
			console.log(reflector.getProperties().join(' | '));
			console.log('***Opportunity.js - # Opportunity.query records returned = ' + data.totalSize);
			var opportunity = data.records;
			deferred.resolve(opportunity);
		}, function(error) {
			console.log('***Opportunity.js - Radius Query Error Function Executing...');
			deferred.reject(error);
			
		});
				
		// return promise to controller to deliver deferred results
		return deferred.promise;
	};
	
	return factory;
	
});