/*
 * Copyright (c) 2012, salesforce.com, inc.
 * All rights reserved.
 * Redistribution and use of this software in source and binary forms, with or
 * without modification, are permitted provided that the following conditions
 * are met:
 * - Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * - Neither the name of salesforce.com, inc. nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission of salesforce.com, inc.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.packt.forcenotify;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;

import org.json.JSONArray;

import android.os.Bundle;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.Toast;

import com.salesforce.androidsdk.app.SalesforceSDKManager;
import com.salesforce.androidsdk.rest.RestClient;
import com.salesforce.androidsdk.rest.RestClient.AsyncRequestCallback;
import com.salesforce.androidsdk.rest.RestRequest;
import com.salesforce.androidsdk.rest.RestResponse;
import com.salesforce.androidsdk.ui.sfnative.SalesforceActivity;

import android.util.Log;
import android.os.AsyncTask;
import com.google.android.gms.gcm.*;
import com.microsoft.windowsazure.messaging.*;
import com.salesforce.androidsdk.rest.RestClient.ClientInfo;


/**
 * Main activity
 */
public class MainActivity extends SalesforceActivity {
	
	private String SENDER_ID = "[Your Google API Project #]";
	private GoogleCloudMessaging gcm;
	private NotificationHub hub;
	private String TAG = "MainActivity.java";
	private String sfUserId = null;


    private RestClient client;
    private ArrayAdapter<String> listAdapter;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		// Setup view
		setContentView(R.layout.main);
		
		// Set up Notification Hub
		Log.d(TAG, "***Initializing Notification Hub...");
		gcm = GoogleCloudMessaging.getInstance(this);
		String connectionString = "[Your Hub Connection String]";
		hub = new NotificationHub("[Your Hub Name]", connectionString, this);

	}
	
	@SuppressWarnings("unchecked")
	private void registerNotificationHubs() {
		// register device with Notification Hub
		new AsyncTask() {
			@Override
			protected Object doInBackground(Object... params) {
				try {
					Log.d(TAG, "***Executing Asynchronous Hub Registration...");
					String regid = gcm.register(SENDER_ID);
					Log.d(TAG, "***GCM Registration ID = " + regid + "...");
					NativeRegistration hubRegistration = hub.register(regid, sfUserId);
					Log.d(TAG, "Hub Registration Details");
					Log.d(TAG, "------------------------");
					Log.d(TAG, "***Hub Registration ID = " + hubRegistration.getRegistrationId() + "...");
					Log.d(TAG, "***Hub Notification Path = " + hubRegistration.getNotificationHubPath() + "...");
					Log.d(TAG, "***Hub GCM Registration ID = " + hubRegistration.getGCMRegistrationId() + "...");
					Log.d(TAG, "***Hub Registration URI = " + hubRegistration.getURI() + "...");
					Log.d(TAG, "***Getting Hub Registration Tags...");
					Log.d(TAG, "***Listing " + hubRegistration.getTags().size() +  " elements...");
					for (String tag : hubRegistration.getTags()) {
						Log.d(TAG, "***Hub Registration Tag = " + tag + "...");
					}
					Log.d(TAG, "***Asynchronous Hub Registration Completed...");
				} catch (Exception e) {
					return e;
				}
				return null;
			}
		}.execute(null, null, null);
		
	}

	
	@Override 
	public void onResume() {
		// Hide everything until we are logged in
		findViewById(R.id.root).setVisibility(View.INVISIBLE);

		// Create list adapter
		listAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, new ArrayList<String>());
		((ListView) findViewById(R.id.contacts_list)).setAdapter(listAdapter);				
		
		super.onResume();
	}		
	
	@Override
	public void onResume(RestClient client) {
        // Keeping reference to rest client
        this.client = client; 

		// Show everything
		findViewById(R.id.root).setVisibility(View.VISIBLE);
		
		Log.d(TAG, "***Getting Logged In User...");
		ClientInfo ci = client.getClientInfo();
		sfUserId = ci.userId.toString();
		String username = ci.username.toString();
		Log.d(TAG, "***Logged In As - " + username + " (ID " + sfUserId + ")...");
		
		Log.d(TAG, "***Registering Notification Hub...");
		registerNotificationHubs();

	}

	/**
	 * Called when "Logout" button is clicked. 
	 * 
	 * @param v
	 */
	public void onLogoutClick(View v) {
		SalesforceSDKManager.getInstance().logout(this);
	}
	
	/**
	 * Called when "Clear" button is clicked. 
	 * 
	 * @param v
	 */
	public void onClearClick(View v) {
		listAdapter.clear();
	}	

	/**
	 * Called when "Fetch Contacts" button is clicked
	 * 
	 * @param v
	 * @throws UnsupportedEncodingException 
	 */
	public void onFetchContactsClick(View v) throws UnsupportedEncodingException {
        sendRequest("SELECT Name FROM Contact");
	}

	/**
	 * Called when "Fetch Accounts" button is clicked
	 * 
	 * @param v
	 * @throws UnsupportedEncodingException 
	 */
	public void onFetchAccountsClick(View v) throws UnsupportedEncodingException {
		sendRequest("SELECT Name FROM Account");
	}	
	
	private void sendRequest(String soql) throws UnsupportedEncodingException {
		RestRequest restRequest = RestRequest.getRequestForQuery(getString(R.string.api_version), soql);

		client.sendAsync(restRequest, new AsyncRequestCallback() {
			@Override
			public void onSuccess(RestRequest request, RestResponse result) {
				try {
					listAdapter.clear();
					JSONArray records = result.asJSONObject().getJSONArray("records");
					for (int i = 0; i < records.length(); i++) {
						listAdapter.add(records.getJSONObject(i).getString("Name"));
					}					
				} catch (Exception e) {
					onError(e);
				}
			}
			
			@Override
			public void onError(Exception exception) {
                Toast.makeText(MainActivity.this,
                               MainActivity.this.getString(SalesforceSDKManager.getInstance().getSalesforceR().stringGenericError(), exception.toString()),
                               Toast.LENGTH_LONG).show();
			}
		});
	}
}
