public with sharing class azureNotificationHubInterface {
	
	// get Azure Access Token
	public string getAzureAccessToken() {
		
		Http h = new Http();
		HttpRequest req = new HttpRequest();
		
		// Getting Security Token
		System.debug(LoggingLevel.DEBUG, '***Setting Variables...');
		string Url = '[Your OAuth WRAP Endpoint]';
		string encodedLogin = EncodingUtil.urlEncode('[Your Default Issuer]', 'UTF-8');
		string encodedPW = EncodingUtil.urlEncode('[Your Default Key]', 'UTF-8');
		
		System.debug(LoggingLevel.DEBUG, '***Setting Endpoint, Request Header, POST Method and Request Body...');
		req.setEndpoint(Url);
		req.setMethod('POST');
		req.setBody('wrap_name=' + encodedLogin + '&wrap_password=' + encodedPW + 
						'&wrap_scope=[Your Notification Hub Namespace Endpoint]');
		req.setHeader('Content-Type', 'application/x-www-form-urlencoded');
		System.debug(LoggingLevel.DEBUG, '***Sending Http Request...');
		System.debug(LoggingLevel.DEBUG, '***Request = ' + req);
		System.debug(LoggingLevel.DEBUG, '***Header = ' + req.getHeader('Content-Type'));
		System.debug(LoggingLevel.DEBUG, '***Request Body = ' + req.getBody());
		HttpResponse res = h.send(req);
		string result = res.getBody();
		System.debug(LoggingLevel.DEBUG, '***RAW Response = ' + result);
		
		// process result to create properly formatted token
		string suffixRemoved = result.split('&')[0];
		string prefixRemoved = suffixRemoved.split('=')[1];
		string decodedToken = EncodingUtil.urlDecode(prefixRemoved, 'UTF-8');
		string finalToken = 'WRAP access_token=\"' + decodedToken + '\"';
		
		System.debug(LoggingLevel.DEBUG, '***Access Token = ' + finalToken);
		
		return finalToken;
	}
		
	// get Azure Hub Registrations
	public List<string> getHubRegistrations(String token) {	
		// Read all Registrations from Notification Hub
		System.debug(LoggingLevel.DEBUG, '***Sending Http Request to Read Hub Registrations...');
		string readUrl = 'https://[Your Notification Hub Namespace Endpoint]/[Your Notification Hub Name]/registrations/?api-version=2013-08';
		Http h = new Http();
		HttpRequest readRequest = new HttpRequest();
		readRequest.setEndpoint(readUrl);
		readRequest.setMethod('GET');
		readRequest.setHeader('Authorization', token);
		readRequest.setHeader('x-ms-version', '2013-08');
		HttpResponse readRes = h.send(readRequest);
		string readResult = readRes.getBody();
		System.debug(LoggingLevel.DEBUG, '***Read Request Response = ' + readResult);
		
		// parse XML and extract registration tags
		XmlStreamReader reader = readRes.getXmlStreamReader();
		List<String> registrationTags = new List<String>();
		
		// use XmlStreamReader to parse response and extract registration tags
		boolean isSafeToGetNextXmlElement = true;
		while (isSafeToGetNextXmlElement) {
			if (reader.getEventType() == XmlTag.START_ELEMENT) {
				if (reader.getLocalName() == 'Tags') {
					System.debug(LoggingLevel.DEBUG, '***Element Name = ' + reader.getLocalName());
					reader.next();
					System.debug(LoggingLevel.DEBUG, '***Element Value = ' + reader.getText());
					registrationTags.add(reader.getText());
				}	
			}
			// check if another node
			if (reader.hasNext()) {
				reader.next();
			} else {
				isSafeToGetNextXmlElement = false;
				break;
			}
			
		}
								
		return registrationTags;
	}
	
	
	// send message to Azure Hub
	public string sendHubMessage(String token, String message, String userTag) { 	
		// Trigger a message to the Notification Hub
		System.debug(LoggingLevel.DEBUG, '***Sending Http Request to Trigger a Notification Hub Message...');
		string messageUrl = 'https://[Your Notification Hub Namespace Endpoint]/[Your Notification Hub Name]/messages/?api-version=2013-08';
		Http h = new Http();
		HttpRequest messageRequest = new HttpRequest();
		messageRequest.setEndpoint(messageUrl);
		messageRequest.setMethod('POST');
		messageRequest.setHeader('Authorization', token);
		messageRequest.setHeader('Content-Type', 'application/json;charset=utf-8');
		messageRequest.setHeader('ServiceBusNotification-Format', 'gcm');
		if (userTag != null) {
			messageRequest.setHeader('ServiceBusNotification-Tags', userTag);
			System.debug(LoggingLevel.DEBUG, '***Setting ServiceBusNotificationTags to ' + userTag + '...');
		}
		messageRequest.setBody('{ "collapse_key": "test_message", "time_to_live": 108, "delay_while_idle": true, "data": { "message": "' + message + '"}}');
		HttpResponse messageRes = h.send(messageRequest);
		string messageResult = messageRes.getBody() + ' ' + messageRes.getStatus() + ' ' + messageRes.getStatusCode();
		System.debug(LoggingLevel.DEBUG, '***Message Request Response = ' + messageResult);
		return messageResult;		
	}

}