public with sharing class forceNotifyController {
    
        // Azure Interface Properties
        private String azureSecurityToken { get; set; }
        private azureNotificationHubInterface notificationHub = new azureNotificationHubInterface();
        
        // User picklist values
        public List<selectOption> registeredDeviceUsers {get; set;}
    
    	// selected value in picklist
    	public string selectedUser {get; set;}
    	
    	// Azure Notification Hub Registration Tags
    	List<String> registrationTags = new List<String>();
    	
    	// message to send
    	public string messageToSend {get; set;}
        
        // constructor
        public forceNotifyController() {
            // get the Azure security token
            azureSecurityToken = notificationHub.getAzureAccessToken();
            System.debug(LoggingLevel.DEBUG, '***Azure Security Token = ' + azureSecurityToken); 
            
            // build dropdown list of users registered with Azure  
        	registeredDeviceUsers = new List<selectOption>();
        	registeredDeviceUsers.add(new selectOption('All','All'));
 			registrationTags = notificationHub.getHubRegistrations(azureSecurityToken);
 			
 			// check if we have any registered users
 			if (registrationTags.size() > 0) {
 			
				List<User> registeredUsers = [SELECT id, Name FROM User WHERE id IN :registrationTags ORDER BY Name];
			
				// remove any duplicates (a user may have multiple devices!)
				List<User> finalDeviceUserList = new List<User>();
				Set<string> idSet = new Set<String>();
				for (User u : registeredUsers) {
					if (idSet.contains(u.id) == false) {
						idSet.add(u.id);
						finalDeviceUserList.add(u);
						System.debug(LoggingLevel.DEBUG, '***Added User To Final List - ' + u.id + ', ' + u.Name);
					}
				}
			
				// add unique users to dropdown list
				for (User finalUser : finalDeviceUserList) {
					registeredDeviceUsers.add(new selectOption(finalUser.id, finalUser.Name));
					System.debug(LoggingLevel.DEBUG, '***Added ' + finalUser.Name + ' to Device User Dropdown List...');
				}
					
				// set initial value for selected User to 'All'
				selectedUser = 'All';
			
 			}
                 
        }
        
        // Send Message
        public PageReference sendNotificationHubMessage() {
                System.debug(LoggingLevel.DEBUG, '***Sending Azure Message - ' + messageToSend);
                
                //TODO: check message <= 255 characters, check if all users selected
                if (messageToSend.length() > 255) {
                	ApexPages.addMessage(new ApexPages.Message(ApexPages.Severity.ERROR,
                									'Message must not be greater than 255 characters'));
                	return null;
                }
                
                // broadcase message to all users
                if (selectedUser == 'All') {
                
                	string sendResult = notificationHub.sendHubMessage(azureSecurityToken, messageToSend, null);
                
                	ApexPages.addMessage(new ApexPages.Message(ApexPages.Severity.INFO,'Message Send Result - ' + sendResult));
                
                	System.debug(LoggingLevel.DEBUG, '***Received Azure Response - ' + sendResult);
                } else {
                	// broadcast to selected user only
                	System.debug(LoggingLevel.DEBUG, '***Sending Targeted Message to - ' + selectedUser);
                	
                	string sendResult = notificationHub.sendHubMessage(azureSecurityToken, messageToSend, selectedUser);
                
                	ApexPages.addMessage(new ApexPages.Message(ApexPages.Severity.INFO,'Message Send Result - ' + sendResult));
                
                	System.debug(LoggingLevel.DEBUG, '***Received Azure Response - ' + sendResult);
                }
                
                return null;
        }

}